@@ -34,6 +34,11 @@ class Required(BasicParam):
     pass
 
 
+class IntegerRequired(BasicParam):
+    """ a required int param """
+    pass
+
+
 class Optional(BasicParam):
     """ an optional param """
     @property
@@ -81,6 +86,8 @@ class ShellParser(argparse.ArgumentParser):
         for param in params:
             if isinstance(param, Required):
                 parser.add_argument(param.label)
+            elif isinstance(param, IntegerRequired):
+                parser.add_argument(param.label, type=int)
             elif isinstance(param, Optional):
                 parser.add_argument(param.label, nargs="?", default="")
             elif isinstance(param, BooleanOptional):
@@ -127,7 +134,7 @@ def ensure_params_with_parser(parser, func):
         try:
             params = parser.parse_args(shlex.split(args[1]))
             return func(args[0], params)
-        except ShellParser.ParserException as ex:
+        except (ShellParser.ParserException, ValueError) as ex:
             print(ex)
     return wrapper
 
@@ -165,7 +172,7 @@ class AugumentedCmd(cmd.Cmd):
         """ the io output object """
         return self._output
 
-    def do_output(self, fmt_str, *params):
+    def show_output(self, fmt_str, *params):
         """ MAX_OUTPUT chars of the last output is available via $? """
         if PYTHON3:
             fmt_str = str(fmt_str)
